package org.objectstyle.dataService;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;

import org.hsqldb.jdbcDriver;
import org.objectstyle.art.Artist;
import org.objectstyle.art.Gallery;
import org.objectstyle.art.Painting;
import org.objectstyle.cayenne.access.DataContext;
import org.objectstyle.cayenne.access.DbGenerator;
import org.objectstyle.cayenne.conn.DriverDataSource;
import org.objectstyle.cayenne.dba.hsqldb.HSQLDBAdapter;
import org.objectstyle.cayenne.map.DataMap;
import org.objectstyle.cayenne.map.MapLoader;
import org.xml.sax.InputSource;

/**
 * Creates database and loads seed data.
 * 
 * @author cris.daniluk@gmail.com
 */
public class DataGenerator extends HttpServlet {

    private String[] seedArtists = new String[] { "Aertsen, Pieter", "Altdorfer, Albrecht", "Altichiero",
            "Angelico, Fra", "Ast, Balthasar van der", "Astruc, Zacharie", "Avercamp, Hendrick ", "Baburen, Dirck van",
            "Bacon, Francis", "Baldung Grien, Hans", "Barocci, Federico", "Bassano (family da Ponte)",
            "Bazille, Frdric", "Bellini, Giovanni", "Beyeren, Abraham van", "Blake, William", "Bosch, Hieronymus",
            "Botticelli, Sandro", "Boucher, Franois", "Boudin, Eugne", "Bouguereau, Adolphe-William", "Bouts, Dirk",
            "Bracquemond, Marie", "Broederlam, Melchior", "Bronzino, Agnolo", "Brown, Ford Madox",
            "Bruegel, Pieter the Elder", "Burgkmair, Hans the Elder", "Burne-Jones, Sir Edward Coley ",
            "Cabanel, Alexandre", "Caillebotte, Gustave", "Canaletto", "Cano, Alonso",
            "Caravaggio, Michelangelo Merisi da", "Caron, Antoine", "Carracci", "Cassatt, Mary", "Cavallini, Pietro",
            "Czanne, Paul", "Chardin, Jean-Baptiste-Simon", "Chase, William Merritt", "Chassriau, Thodore",
            "Christus, Petrus", "Cima da Conegliano, Giovanni Battista", "Clouet", "Cole, Thomas", "Constable, John",
            "Copley, John Singleton", "Cornell, Joseph", "Corot, Jean-Baptiste-Camille", "Correggio",
            "Cosimo, Piero di", "Courbet, Gustave", "Couture, Thomas", "Cranach, Lucas the Elder", "Cropsey, Jasper",
            "Cuyp ", "Daumier, Honor", "David, Gerard", "David, Jacques-Louis", "Davis, Stuart", "Degas, Edgar",
            "Delacroix, Eugne", "Desportes, Alexandre-Franois", "Dor, Gustave", "Dossi, Dosso", "Drer, Albrecht",
            "Dyck, Sir Anthony van ", "Eakins, Thomas", "Elsheimer, Adam", "Eyck, Jan van ", "Fantin-Latour, Henri",
            "Feke, Robert", "Flmalle, Master of (Campin, Robert)", "Fouquet, Jean", "Fragonard, Jean-Honor",
            "Freud, Lucian", "Friedrich, Caspar David", "Fuseli, Henry ", "Gainsborough, Thomas",
            "Gauguin, (Eugne-Henri-) Paul", "Gelder, Aert de", "Gentile da Fabriano", "Gricault, Thodore",
            "Grme, Jean-Lon", "Ghirlandaio, Domenico", "Giordano, Luca", "Giotto di Bondone", "Goes, Hugo van der",
            "Gogh, Vincent van", "Gonalves, Nuo", "Goya, Francisco de", "Greco, El", "Greuze, Jean-Baptiste",
            "Gris, Juan", "Gros, Antoine-Jean", "Grnewald, Matthias", "Guardi, Francesco",
            "Guillaumin, Jean-Baptiste Armand ", "Hals, Frans", "Heda, Willem Claesz.", "Heem, Jan Davidsz. de",
            "Hilliard, Nicholas", "Hiroshige, Ando", "Hobbema, Meindert", "Hockney, David", "Hokusai, Katsushika",
            "Holbein, Hans the Younger", "Hooch, Pieter de", "Hopper, Edward ", "Ingres, Jean-Auguste-Dominique ",
            "Jongkind, Johan-Barthold", "Joos van Wassenhove ", "Kandinsky, Wassily", "Kiefer, Anselm",
            "Kitaj, Ron B.", "Klee, Paul", "Klimt, Gustav ", "La Hire, Laurent de", "La Fosse, Charles de",
            "Lancret, Nicolas", "Largillire, Nicolas de", "Le Brun, Charles", "Legros, Alphonse", "Leonardo da Vinci",
            "Lorrain, Claude ", "Malevich, Kasimir", "Manet, Edouard", "Mantegna, Andrea", "Marc, Franz",
            "Martini, Simone", "Matisse, Henri", "Maureau, Alphonse", "Memling (or Memlinc), Hans", "Michelangelo",
            "Millet, Jean-Franois", "Modigliani, Amedeo", "Mondrian, Piet", "Monet, Claude", "Moreau, Gustave",
            "Morisot, Berthe", "Munch, Edvard", "Murillo, Bartolom Esteban ", "Partridge, Nehemiah", "Patel, Pierre",
            "Piero della Francesca", "Piette, Ludovic", "Pissarro, Camille", "Pollock, Jackson",
            "Puvis de Chavannes, Pierre ", "Raphael", "Redon, Odilon", "Rembrandt", "Renoir, Pierre-Auguste",
            "Rodin, Auguste", "Rossetti, Dante Gabriel", "Rousseau, Henri", "Rubens, Peter Paul ",
            "Sargent, John Singer", "Schiele, Egon", "Seurat, Georges", "Shaw, Joshua", "Signac, Paul",
            "Sisley, Alfred ", "Tanning, Dorothea", "Tanguy, Yves", "Tenniel, Sir John", "Tiepolo, Giambattista",
            "Tintoretto", "Tissot, James", "Titian", "Toulouse-Lautrec, Henri de", "Trevino, Jesse",
            "Troy, Jean-Franois de", "Turner, Joseph Mallord William ", "Uccello, Paolo ", "Velzquez, Diego",
            "Vermeer, Jan", "Veronese, Paolo", "Waterhouse, John William", "Watteau, Jean-Antoine", "West, Benjamin",
            "Weyden, Rogier van der", "Whistler, James Abbott McNeill", "Winstanley, William", "Wright, Joseph ",
            "Zurbarn, Francisco de " };
    
    private String[] seedPaintings = new String[] { "Garon  la pipe", "Dr. Gachet",
            "Bal au moulin de la Galette, Montmartre", "Irises", "Massacre of the Innocents", "Les Noces de Pierrette",
            "Portrait de l'artiste sans barbe", "Rideau, Cruchon et Compotier", "Femme aux Bras Croiss" };
    
    private String[] seedGalleries = new String[] { "The Art Institute of Chicago", "The British Museum, London",
            "Brooklyn Museum of Art, New York", "Budapest Kunsthalle", "Carnegie Museum of Art, Pittsburgh",
            "Contemporary Arts Museum, Houston", "Dallas Museum of Art", "Denver Art Museum",
            "Dia Center for the Arts, New York", "The Fine Arts Museums of San Francisco",
            "Hamburger Kunsthalle, Germany", "Harvard Museums, Cambridge, Massachusetts",
            "International Center of Photography, New York", "The J. Paul Getty Museum, Los Angeles",
            "Kunsthaus Zrich", "Kunsthistorisches Museum, Vienna", "Muse d'Art Contemporain de Montral",
            "Muse du Louvre, Paris", "Museum of Contemporary Art, Chicago", "Museum of Contemporary Art, Los Angeles",
            "Museum of Contemporary Art, San Diego", "The Museum of Modern Art, New York",
            "The Museum of the City of New York", "National Gallery of Art, Washington",
            "National Gallery of Australia, Canberra", "National Gallery of Canada, Ottawa",
            "National Gallery, London", "National Museum of American Art, Washington",
            "The National Museum of Western Art, Tokyo", "New Orleans Museum of Art", "Philadelphia Museum of Art",
            "Pushkin Museum, Moscow", "Rijksmuseum, Amsterdam", "Royal Academy of Arts, London",
            "San Francisco Museum of Modern Art", "Solomon R. Guggenheim Museum, New York",
            "Staatliche Museen, Berlin", "Tate Gallery, London", "Berkeley Art Museum and Pacific Film Archive",
            "Victoria and Albert Museum, London", "Yale University Art Gallery, New Haven" };

    public void init() throws ServletException {
        final String URL = "jdbc:hsqldb:mem:genericDataService";

        DriverDataSource dataSource = new DriverDataSource(new jdbcDriver(), URL, "sa", "");
        DataMap dataMap;
        try {
            dataMap = new MapLoader().loadDataMap(new InputSource(getClass().getClassLoader().getResourceAsStream(
                    "testmap.map.xml")));

            DbGenerator generator = new DbGenerator(new HSQLDBAdapter(), dataMap);
            generator.setShouldCreateFKConstraints(false);
            generator.setShouldCreatePKSupport(true);
            generator.setShouldCreateTables(true);
            generator.setShouldDropPKSupport(false);
            generator.setShouldDropTables(false);

            generator.runGenerator(dataSource);

        } catch (Exception e) {
            throw new ServletException(e.getMessage(), e);
        }

        DataContext context = DataContext.createDataContext();

        List artistList = new ArrayList();
        for (int i = 0; i < seedArtists.length; i++) {
            Artist artist = (Artist) context.createAndRegisterNewObject(Artist.class);
            artist.setArtistName(seedArtists[i]);
            artist.setDateOfBirth(new Date());
            artistList.add(artist);
        }
        
        List galleryList = new ArrayList();
        for (int i = 0; i < seedGalleries.length; i++) {
            Gallery gallery = (Gallery) context.createAndRegisterNewObject(Gallery.class);
            gallery.setGalleryName(seedGalleries[i]);
            galleryList.add(gallery);
        }
        
        for (int i = 0; i < seedPaintings.length; i++) {
            Painting painting = (Painting) context.createAndRegisterNewObject(Painting.class);
            painting.setPaintingTitle(seedPaintings[i]);
            painting.setPaintingDescription("Description");
            painting.setEstimatedPrice(new BigDecimal(500000));
            
            painting.setToArtist((Artist) artistList.get(i % artistList.size()));
            painting.setToGallery((Gallery) galleryList.get(i % galleryList.size()));
        }
        
        context.commitChanges();

    }
}
