package org.objectstyle.dataService.struts;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.objectstyle.cayenne.access.DataContext;
import org.objectstyle.cayenne.exp.Expression;
import org.objectstyle.cayenne.query.Ordering;
import org.objectstyle.dataService.GenericDataService;

/**
 * @author cris.daniluk@gmail.com
 */
public abstract class AbstractDataServiceAction extends Action {

    public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) throws Exception {
        
        DataContext context = DataContext.getThreadDataContext();
        
        AbstractDataServiceActionForm dataForm = (AbstractDataServiceActionForm) form;

        GenericDataService dataService = null;
        if (getMappingFile() != null) {
            dataService = new GenericDataService(context, getMappingFile());
        } else {
            dataService = new GenericDataService(context);
        }

        Expression expression = null;
        if (StringUtils.isNotBlank(dataForm.getExpression())) {

            Map expressionMap = getExpressionMap();

            if (expressionMap == null) {
                expression = Expression.fromString(dataForm.getExpression());
            } else if (expressionMap.size() == 1) {
                expression = null;
            } else {
                expression = (Expression) expressionMap.get(dataForm.getExpression());
            }
        }

        List sortList = null;
        if (StringUtils.isNotBlank(dataForm.getOrdering())) {
            
            sortList = new ArrayList();

            String[] orderings = dataForm.getOrdering().split(",");

            for (int i = 0; i < orderings.length; i++) {

                boolean ascending = true;
                char firstChar = orderings[i].charAt(0);

                if (firstChar == '-') {
                    orderings[i] = orderings[i].substring(1);
                    ascending = false;
                } else {
                    orderings[i] = orderings[i].substring(1);
                }

                sortList.add(new Ordering(orderings[i], ascending));
            }
        }

        String resultSet = null;
        boolean filterInMemory = true;
        if (dataForm.getFilterInMemory() != null) {
            filterInMemory = Boolean.valueOf(dataForm.getFilterInMemory()).booleanValue();
        }

        if (StringUtils.isNotBlank(dataForm.getObjectType())) {

            resultSet = dataService.returnObjects((Class) getObjectMap().get(dataForm.getObjectType()), expression,
                    sortList, filterInMemory);
        } else if (StringUtils.isNotBlank(dataForm.getQuery())) {

            resultSet = dataService.returnStoredQuery(dataForm.getQuery(), expression, sortList, filterInMemory);

        } else {
            // Invalid query
            throw new Exception("Invalid query type");
        }
        
        response.setContentType("application/xml");
        PrintWriter out = response.getWriter();
        out.print(resultSet);
        
        return null;
    }
    
    /**
     * Provides a mapping of objects to Cayenne objects. This ensures that data
     * objects that are not intended to be directly exposed to the browser
     * and/or user cannot be accessed. This method must be overridden.
     * 
     * @return
     */
    protected abstract Map getObjectMap();

    /**
     * This method can be overridden to validate filters, providing a mechanism
     * to prevent potentially abusive queries, etc.
     * 
     * @see org.objectstyle.cayenne.exp.Expression
     * @return
     */
    protected boolean expressionValidator() {
        return true;
    }

    /**
     * This method can be overriden to provide a list of expressions that may be
     * run. If you are concerned about the security of allowing users to execute
     * arbitrary expressions, use this method. To disable support for
     * expressions, you can override this method and return an empty map. If
     * this method is not extended, all expressions will be permitted.
     * 
     * @see org.objectstyle.cayenne.exp.Expression
     * @return Map of Strings to Expressions
     */
    protected Map getExpressionMap() {
        return Collections.EMPTY_MAP;
    }
    
    /**
     * This method can be overridden to specify an XML mapping file. By default, no mapping file is used.
     * @return null
     */
    protected String getMappingFile() {
        return null;
    }

}
